static const char* op_c_source =
"/* This file is an image processing operation for GEGL                        \n"
" *                                                                            \n"
" * GEGL is free software; you can redistribute it and/or                      \n"
" * modify it under the terms of the GNU Lesser General Public                 \n"
" * License as published by the Free Software Foundation; either               \n"
" * version 3 of the License, or (at your option) any later version.           \n"
" *                                                                            \n"
" * GEGL is distributed in the hope that it will be useful,                    \n"
" * but WITHOUT ANY WARRANTY; without even the implied warranty of             \n"
" * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU          \n"
" * Lesser General Public License for more details.                            \n"
" *                                                                            \n"
" * You should have received a copy of the GNU Lesser General Public           \n"
" * License along with GEGL; if not, see <http://www.gnu.org/licenses/>.       \n"
" *                                                                            \n"
" * Copyright (C) 1995 Spencer Kimball and Peter Mattis                        \n"
" *                                                                            \n"
" * This plug-in uses the algorithm described by John Schlag, \"Fast           \n"
" * Embossing Effects on Raster Image Data\" in Graphics Gems IV (ISBN         \n"
" * 0-12-336155-9).  It takes a grayscale image to be applied as a             \n"
" * bump-map to another image, producing a nice embossing effect.              \n"
" *                                                                            \n"
" * Ported to gegl by Dov Grobgeld <dov.grobgeld@gmail.com>                    \n"
" *                                                                            \n"
" * ToDo:                                                                      \n"
" *                                                                            \n"
" *  - Make lut table size configurable?                                       \n"
" *  - Support and y-offset tiling of the bump map.                            \n"
" *  - Make an opencl version.                                                 \n"
" *  - Don't \"upgrade\" Y,YA and RGB images to RGBA as it is a waste of time. \n"
" */                                                                           \n"
"                                                                              \n"
"#include \"config.h\"                                                         \n"
"                                                                              \n"
"#include <glib/gi18n-lib.h>                                                   \n"
"                                                                              \n"
"#ifdef GEGL_PROPERTIES                                                        \n"
"                                                                              \n"
"enum_start (gegl_bump_map_type)                                               \n"
"  enum_value (GEGL_BUMP_MAP_TYPE_LINEAR,     \"linear\",     N_(\"Linear\"))  \n"
"  enum_value (GEGL_BUMP_MAP_TYPE_SPHERICAL,  \"spherical\",  N_(\"Spherical\"))\n"
"  enum_value (GEGL_BUMP_MAP_TYPE_SINUSOIDAL, \"sinusodial\", N_(\"Sinusoidal\"))\n"
"enum_end (GeglBumpMapType)                                                    \n"
"                                                                              \n"
"property_enum (type, _(\"Type\"), GeglBumpMapType, gegl_bump_map_type,        \n"
"               GEGL_BUMP_MAP_TYPE_LINEAR)                                     \n"
"  description (_(\"Type of map\"))                                            \n"
"                                                                              \n"
"property_boolean (compensate, _(\"Compensate\"), TRUE)                        \n"
"  description (_(\"Compensate for darkening\"))                               \n"
"                                                                              \n"
"property_boolean (invert, _(\"Invert\"), FALSE)                               \n"
"  description (_(\"Invert bumpmap\"))                                         \n"
"                                                                              \n"
"property_boolean (tiled, _(\"Tiled\"), FALSE)                                 \n"
"  description (_(\"Tiled bumpmap\"))                                          \n"
"                                                                              \n"
"property_double  (azimuth, _(\"Azimuth\"), 135.0)                             \n"
"  value_range (0.0, 360.0)                                                    \n"
"  ui_meta     (\"unit\", \"degree\")                                          \n"
"                                                                              \n"
"property_double  (elevation, _(\"Elevation\"), 45.0)                          \n"
"  value_range (0.0, 360.0)                                                    \n"
"                                                                              \n"
"property_double  (depth, _(\"Depth\"), 0.005)                                 \n"
"  value_range (0.0005, 100)                                                   \n"
"                                                                              \n"
"property_int  (offset_x, _(\"Offset X\"), -1000.0)                            \n"
"  value_range (-20000, 20000)                                                 \n"
"  ui_range (-1000.0, 1000.0)                                                  \n"
"  ui_meta  (\"axis\", \"x\")                                                  \n"
"  ui_meta  (\"unit\", \"pixel-coordinate\")                                   \n"
"                                                                              \n"
"property_int  (offset_y, _(\"Offset Y\"), -1000.0)                            \n"
"  value_range (-20000, 20000)                                                 \n"
"  ui_range (-1000.0, 1000.0)                                                  \n"
"  ui_meta  (\"axis\", \"y\")                                                  \n"
"  ui_meta  (\"unit\", \"pixel-coordinate\")                                   \n"
"                                                                              \n"
"property_double  (waterlevel, _(\"Waterlevel\"), 0.0)                         \n"
"  description(_(\"Level that full transparency should represent\"))           \n"
"  value_range (0.0, 1.0)                                                      \n"
"                                                                              \n"
"property_double  (ambient, _(\"Ambient lighting factor\"), 0.0)               \n"
"  value_range (0.0, 1.0)                                                      \n"
"                                                                              \n"
"#else                                                                         \n"
"                                                                              \n"
"#define GEGL_OP_COMPOSER                                                      \n"
"#define GEGL_OP_C_SOURCE bump-map.c                                           \n"
"                                                                              \n"
"#include \"gegl-op.h\"                                                        \n"
"#include <math.h>                                                             \n"
"                                                                              \n"
"/***** Macros *****/                                                          \n"
"                                                                              \n"
"// Should the LUT table be a configurable property? Should it                 \n"
"// be interpolated instead of by a plain lookup?                              \n"
"#define LUT_TABLE_SIZE 8192                                                   \n"
"                                                                              \n"
"#define MOD(x, y) \\                                                          \n"
"  ((x) < 0 ? ((y) - 1 - ((y) - 1 - (x)) % (y)) : (x) % (y))                   \n"
"                                                                              \n"
"typedef struct                                                                \n"
"{                                                                             \n"
"  gdouble lx, ly;       /* X and Y components of light vector */              \n"
"  gdouble nz2, nzlz;    /* nz^2, nz*lz */                                     \n"
"  gdouble background;   /* Shade for vertical normals */                      \n"
"  gdouble compensation; /* Background compensation */                         \n"
"  gfloat lut[LUT_TABLE_SIZE];    /* Look-up table for modes - should be made interpolated*/\n"
"} bumpmap_params_t;                                                           \n"
"                                                                              \n"
"static void                                                                   \n"
"bumpmap_setup_calc (GeglProperties     *o,                                    \n"
"                    bumpmap_params_t *params)                                 \n"
"{                                                                             \n"
"  /* Convert to radians */                                                    \n"
"  const gdouble azimuth   = G_PI * o->azimuth / 180.0;                        \n"
"  const gdouble elevation = G_PI * o->elevation / 180.0;                      \n"
"                                                                              \n"
"  gdouble lz, nz;                                                             \n"
"  gint i;                                                                     \n"
"                                                                              \n"
"  /* Calculate the light vector */                                            \n"
"  params->lx = cos (azimuth) * cos (elevation);                               \n"
"  params->ly = sin (azimuth) * cos (elevation);                               \n"
"  lz         = sin (elevation);                                               \n"
"                                                                              \n"
"  /* Calculate constant Z component of surface normal */                      \n"
"  /*              (depth may be 0 if non-interactive) */                      \n"
"  nz           = 6.0 / MAX (o->depth, 0.001);                                 \n"
"  params->nz2  = nz * nz;                                                     \n"
"  params->nzlz = nz * lz;                                                     \n"
"                                                                              \n"
"  /* Optimize for vertical normals */                                         \n"
"  params->background = lz;                                                    \n"
"                                                                              \n"
"  /* Calculate darkness compensation factor */                                \n"
"  params->compensation = sin (elevation);                                     \n"
"                                                                              \n"
"  /* Create look-up table for map type */                                     \n"
"  for (i = 0; i < LUT_TABLE_SIZE; i++)                                        \n"
"    {                                                                         \n"
"      gdouble n;                                                              \n"
"                                                                              \n"
"      switch (o->type)                                                        \n"
"        {                                                                     \n"
"        case GEGL_BUMP_MAP_TYPE_SPHERICAL:                                    \n"
"          n = 1.0 - 1.0*i / LUT_TABLE_SIZE;                                   \n"
"          params->lut[i] = sqrt(1.0 - n * n) + 0.5;                           \n"
"          break;                                                              \n"
"                                                                              \n"
"        case GEGL_BUMP_MAP_TYPE_SINUSOIDAL:                                   \n"
"          n = 1.0 * i / (LUT_TABLE_SIZE-1);                                   \n"
"          params->lut[i] = (sin((-G_PI / 2.0) + G_PI * n) + 1.0) / 2.0 + 0.5; \n"
"          break;                                                              \n"
"                                                                              \n"
"        case GEGL_BUMP_MAP_TYPE_LINEAR:                                       \n"
"        default:                                                              \n"
"          params->lut[i] = 1.0*i/(LUT_TABLE_SIZE-1);                          \n"
"        }                                                                     \n"
"                                                                              \n"
"      if (o->invert)                                                          \n"
"        params->lut[i] = 1.0 - params->lut[i];                                \n"
"    }                                                                         \n"
"}                                                                             \n"
"                                                                              \n"
"static void                                                                   \n"
"bumpmap_row (const gfloat     *src,                                           \n"
"             gfloat           *dest,                                          \n"
"             gint              width,                                         \n"
"             gint              nchannels,                                     \n"
"             gint              has_alpha,                                     \n"
"             const gfloat     *bm_row1,                                       \n"
"             const gfloat     *bm_row2,                                       \n"
"             const gfloat     *bm_row3,                                       \n"
"             gint              bm_width,                                      \n"
"             gint              bm_xofs,                                       \n"
"             gboolean          row_in_bumpmap,                                \n"
"             GeglProperties       *o,                                         \n"
"             bumpmap_params_t *params)                                        \n"
"{                                                                             \n"
"  gint xofs1, xofs2, xofs3;                                                   \n"
"  gint x, k;                                                                  \n"
"  gdouble result;                                                             \n"
"                                                                              \n"
"  xofs2 = MOD (bm_xofs,bm_width);                                             \n"
"                                                                              \n"
"  for (x = 0; x < width; x++)                                                 \n"
"    {                                                                         \n"
"      double shade;                                                           \n"
"      double nx, ny;                                                          \n"
"                                                                              \n"
"      /* Calculate surface normal from bump map */                            \n"
"                                                                              \n"
"      if (o->tiled || (row_in_bumpmap &&                                      \n"
"                       x >= - bm_xofs && x < - bm_xofs + bm_width))           \n"
"        {                                                                     \n"
"          if (o->tiled)                                                       \n"
"            {                                                                 \n"
"              xofs1 = MOD (xofs2 - 1, bm_width);                              \n"
"              xofs3 = MOD (xofs2 + 1, bm_width);                              \n"
"            }                                                                 \n"
"          else                                                                \n"
"            {                                                                 \n"
"              xofs1 = CLAMP (xofs2 - 1, 0, bm_width - 1);                     \n"
"              xofs3 = CLAMP (xofs2 + 1, 0, bm_width - 1);                     \n"
"            }                                                                 \n"
"                                                                              \n"
"          nx = (bm_row1[xofs1] + bm_row2[xofs1] + bm_row3[xofs1] -            \n"
"                bm_row1[xofs3] - bm_row2[xofs3] - bm_row3[xofs3]);            \n"
"          ny = (bm_row3[xofs1] + bm_row3[xofs2] + bm_row3[xofs3] -            \n"
"                bm_row1[xofs1] - bm_row1[xofs2] - bm_row1[xofs3]);            \n"
"        }                                                                     \n"
"      else                                                                    \n"
"        {                                                                     \n"
"          nx = ny = 0;                                                        \n"
"        }                                                                     \n"
"                                                                              \n"
"      if ((nx == 0) && (ny == 0))                                             \n"
"        {                                                                     \n"
"          shade = params->background;                                         \n"
"        }                                                                     \n"
"      else                                                                    \n"
"        {                                                                     \n"
"          double ndotl = nx * params->lx + ny * params->ly + params->nzlz;    \n"
"                                                                              \n"
"          if (ndotl < 0)                                                      \n"
"            {                                                                 \n"
"              shade = params->compensation * o->ambient;                      \n"
"            }                                                                 \n"
"          else                                                                \n"
"            {                                                                 \n"
"              double pre_shade = ndotl / sqrt (nx * nx + ny * ny + params->nz2);\n"
"                                                                              \n"
"              shade = pre_shade + MAX(0, (params->compensation - pre_shade)) *\n"
"                o->ambient;                                                   \n"
"            }                                                                 \n"
"        }                                                                     \n"
"                                                                              \n"
"      /* Paint */                                                             \n"
"      if (o->compensate)                                                      \n"
"        {                                                                     \n"
"          for (k = nchannels-has_alpha; k; k--)                               \n"
"            {                                                                 \n"
"              result  = (*src++ * shade) / params->compensation;              \n"
"              *dest++ = result;                                               \n"
"            }                                                                 \n"
"        }                                                                     \n"
"      else                                                                    \n"
"        {                                                                     \n"
"          for (k = nchannels-has_alpha; k; k--)                               \n"
"            *dest++ = *src++ * shade;                                         \n"
"        }                                                                     \n"
"                                                                              \n"
"      if (has_alpha)                                                          \n"
"        *dest++ = *src++;                                                     \n"
"                                                                              \n"
"      /* Next pixel */                                                        \n"
"      if (++xofs2 == bm_width)                                                \n"
"        xofs2 = 0;                                                            \n"
"    }                                                                         \n"
"}                                                                             \n"
"                                                                              \n"
"// Map the FIFO bumpmap row according to the lookup table.                    \n"
"static void                                                                   \n"
"bumpmap_convert_row (gfloat       *row,                                       \n"
"                     gint          width,                                     \n"
"                     const gfloat  *lut,                                      \n"
"                     double        waterlevel)                                \n"
"{                                                                             \n"
"  gfloat *p = row;                                                            \n"
"                                                                              \n"
"  for (; width; width--)                                                      \n"
"    {                                                                         \n"
"      int lut_idx = CLAMP((int)(waterlevel * (LUT_TABLE_SIZE-1)               \n"
"                                + *p *(LUT_TABLE_SIZE-1)*(1.0-waterlevel)),0,(LUT_TABLE_SIZE-1));\n"
"      *p++ = lut[lut_idx];                                                    \n"
"    }                                                                         \n"
"}                                                                             \n"
"                                                                              \n"
"static void                                                                   \n"
"prepare (GeglOperation *operation)                                            \n"
"{                                                                             \n"
"  gegl_operation_set_format (operation, \"input\",  babl_format (\"RGBA float\"));\n"
"  gegl_operation_set_format (operation, \"aux\",    babl_format (\"Y float\"));\n"
"  gegl_operation_set_format (operation, \"output\", babl_format (\"RGBA float\"));\n"
"}                                                                             \n"
"                                                                              \n"
"static gboolean                                                               \n"
"process (GeglOperation       *operation,                                      \n"
"         GeglBuffer          *input,                                          \n"
"         GeglBuffer          *aux,                                            \n"
"         GeglBuffer          *output,                                         \n"
"         const GeglRectangle *rect,                                           \n"
"         gint                 level)                                          \n"
"{                                                                             \n"
"  GeglProperties *o = GEGL_PROPERTIES (operation);                            \n"
"  gfloat  *src_buf, *dst_buf, *bm_row1, *bm_row2, *bm_row3, *src_row, *dest_row, *bm_tmprow;\n"
"  const Babl *format = gegl_operation_get_format (operation, \"output\");     \n"
"  gint channels = babl_format_get_n_components (format);                      \n"
"  bumpmap_params_t params;                                                    \n"
"  gint yofs1, yofs2, yofs3;                                                   \n"
"  gint row_stride;                                                            \n"
"  gint row, y;                                                                \n"
"  gint bm_width, bm_height;                                                   \n"
"  gint slice_thickness = 32;                                                  \n"
"  gboolean first_time = TRUE;                                                 \n"
"                                                                              \n"
"  // This should be made more sophisticated                                   \n"
"  int has_alpha = (channels == 4) || (channels == 2);                         \n"
"                                                                              \n"
"  if (! aux)                                                                  \n"
"    return FALSE;                                                             \n"
"                                                                              \n"
"  bm_width = gegl_buffer_get_width (aux);                                     \n"
"  bm_height = gegl_buffer_get_height (aux);                                   \n"
"                                                                              \n"
"  src_buf    = g_new0 (gfloat, rect->width * slice_thickness * channels);     \n"
"  dst_buf    = g_new0 (gfloat, rect->width * slice_thickness * channels);     \n"
"                                                                              \n"
"  bumpmap_setup_calc (o, &params);                                            \n"
"                                                                              \n"
"  /* Initialize offsets */                                                    \n"
"  if (o->tiled)                                                               \n"
"    {                                                                         \n"
"      yofs2 = MOD (o->offset_y, bm_height);                                   \n"
"      yofs1 = MOD (yofs2 - 1, bm_height);                                     \n"
"      yofs3 = MOD (yofs2 + 1, bm_height);                                     \n"
"    }                                                                         \n"
"  else                                                                        \n"
"    {                                                                         \n"
"      yofs2 = o->offset_y;                                                    \n"
"      yofs1 = yofs2 - 1;                                                      \n"
"      yofs3 = yofs2 + 1;                                                      \n"
"    }                                                                         \n"
"                                                                              \n"
"  /* Initialize three line fifo buffers */                                    \n"
"  bm_row1 = g_new (gfloat, bm_width);                                         \n"
"  bm_row2 = g_new (gfloat, bm_width);                                         \n"
"  bm_row3 = g_new (gfloat, bm_width);                                         \n"
"                                                                              \n"
"  // The source and destination row stride in floats                          \n"
"  row_stride = rect->width*channels;                                          \n"
"                                                                              \n"
"  // Process the input buffer one slice at a time, but the bumpmap one row at a time.\n"
"  for (row=rect->y; row < rect->y+rect->height; row+= slice_thickness)        \n"
"    {                                                                         \n"
"      GeglRectangle rect_slice, bm_rect;                                      \n"
"      rect_slice.x = rect->x;                                                 \n"
"      rect_slice.width = rect->width;                                         \n"
"      rect_slice.y = rect->y+row;                                             \n"
"      rect_slice.height = MIN (slice_thickness, rect->height-row);            \n"
"      gegl_buffer_get (input, &rect_slice, 1.0,                               \n"
"                       babl_format (\"RGBA float\"), src_buf,                 \n"
"                       GEGL_AUTO_ROWSTRIDE, GEGL_ABYSS_NONE);                 \n"
"                                                                              \n"
"      // Get the bumpmap one row at a time. The following values are constant \n"
"      // in the bumpmap buffer access.                                        \n"
"      bm_rect.x = 0;                                                          \n"
"      bm_rect.width = bm_width;                                               \n"
"      bm_rect.height = 1;                                                     \n"
"                                                                              \n"
"      for (y = 0; y < rect_slice.height; y++)                                 \n"
"        {                                                                     \n"
"          gboolean row_in_bumpmap = (yofs2 > 0 && yofs2 < bm_height);         \n"
"                                                                              \n"
"          // Fill in the three rows FIFO the first time we are inside the bumpmap.\n"
"          if (row_in_bumpmap && first_time)                                   \n"
"            {                                                                 \n"
"              first_time = FALSE;                                             \n"
"                                                                              \n"
"              bm_rect.y = yofs1;                                              \n"
"              gegl_buffer_get (aux, &bm_rect, 1.0,                            \n"
"                               babl_format (\"Y float\"), bm_row1,            \n"
"                               GEGL_AUTO_ROWSTRIDE, GEGL_ABYSS_NONE);         \n"
"              bm_rect.y = yofs2;                                              \n"
"              gegl_buffer_get (aux, &bm_rect, 1.0,                            \n"
"                               babl_format (\"Y float\"), bm_row2,            \n"
"                               GEGL_AUTO_ROWSTRIDE, GEGL_ABYSS_NONE);         \n"
"              bm_rect.y = yofs3;                                              \n"
"              gegl_buffer_get (aux, &bm_rect, 1.0,                            \n"
"                               babl_format (\"Y float\"), bm_row3,            \n"
"                               GEGL_AUTO_ROWSTRIDE, GEGL_ABYSS_NONE);         \n"
"                                                                              \n"
"              bumpmap_convert_row (bm_row1, bm_width, params.lut, o->waterlevel);\n"
"              bumpmap_convert_row (bm_row2, bm_width, params.lut, o->waterlevel);\n"
"              bumpmap_convert_row (bm_row3, bm_width, params.lut, o->waterlevel);\n"
"            }                                                                 \n"
"                                                                              \n"
"          src_row = src_buf + y * row_stride;                                 \n"
"          dest_row = dst_buf + y * row_stride;                                \n"
"                                                                              \n"
"          bumpmap_row (src_row, dest_row,                                     \n"
"                       rect->width,                                           \n"
"                       channels,                                              \n"
"                       has_alpha,                                             \n"
"                       bm_row1, bm_row2, bm_row3,                             \n"
"                       bm_width,                                              \n"
"                       o->offset_x,                                           \n"
"                       row_in_bumpmap,                                        \n"
"                       o,                                                     \n"
"                       &params);                                              \n"
"                                                                              \n"
"          if (!first_time)                                                    \n"
"            {                                                                 \n"
"              /* Next line */                                                 \n"
"              bm_tmprow = bm_row1;                                            \n"
"              bm_row1   = bm_row2;                                            \n"
"              bm_row2   = bm_row3;                                            \n"
"              bm_row3   = bm_tmprow;                                          \n"
"            }                                                                 \n"
"                                                                              \n"
"          if (++yofs2 == bm_height && o->tiled)                               \n"
"            yofs2 = 0;                                                        \n"
"                                                                              \n"
"          if (o->tiled)                                                       \n"
"            yofs3 = MOD (yofs2 + 1, bm_height);                               \n"
"          else                                                                \n"
"            yofs3 = CLAMP (yofs2 + 1, 0, bm_height);                          \n"
"                                                                              \n"
"          if (!first_time)                                                    \n"
"            {                                                                 \n"
"              bm_rect.y = yofs3;                                              \n"
"              gegl_buffer_get (aux, &bm_rect, 1.0, babl_format (\"Y float\"), bm_row3,\n"
"                               GEGL_AUTO_ROWSTRIDE, GEGL_ABYSS_NONE);         \n"
"              bumpmap_convert_row (bm_row3, bm_width, params.lut, o->waterlevel);\n"
"            }                                                                 \n"
"        }                                                                     \n"
"                                                                              \n"
"      gegl_buffer_set (output, &rect_slice, 0,                                \n"
"                       babl_format (\"RGBA float\"), dst_buf,                 \n"
"                       GEGL_AUTO_ROWSTRIDE);                                  \n"
"    }                                                                         \n"
"                                                                              \n"
"  g_free (bm_row1);                                                           \n"
"  g_free (bm_row2);                                                           \n"
"  g_free (bm_row3);                                                           \n"
"                                                                              \n"
"  g_free (dst_buf);                                                           \n"
"  g_free (src_buf);                                                           \n"
"                                                                              \n"
"  return TRUE;                                                                \n"
"}                                                                             \n"
"                                                                              \n"
"static GeglRectangle                                                          \n"
"get_bounding_box (GeglOperation *self)                                        \n"
"{                                                                             \n"
"  GeglRectangle  result  = { 0, 0, 0, 0 };                                    \n"
"  GeglRectangle *in_rect = gegl_operation_source_get_bounding_box (self, \"input\");\n"
"                                                                              \n"
"  if (! in_rect)                                                              \n"
"    return result;                                                            \n"
"                                                                              \n"
"  return *in_rect;                                                            \n"
"}                                                                             \n"
"                                                                              \n"
"static void                                                                   \n"
"gegl_op_class_init (GeglOpClass *klass)                                       \n"
"{                                                                             \n"
"  GeglOperationClass         *operation_class;                                \n"
"  GeglOperationComposerClass *composer_class;                                 \n"
"                                                                              \n"
"  operation_class = GEGL_OPERATION_CLASS (klass);                             \n"
"  composer_class  = GEGL_OPERATION_COMPOSER_CLASS (klass);                    \n"
"                                                                              \n"
"  operation_class->prepare          = prepare;                                \n"
"  operation_class->get_bounding_box = get_bounding_box;                       \n"
"  composer_class->process           = process;                                \n"
"                                                                              \n"
"  gegl_operation_class_set_keys (operation_class,                             \n"
"    \"name\",        \"gegl:bump-map\",                                       \n"
"    \"categories\",  \"light\",                                               \n"
"    \"title\",       _(\"Bump Map\"),                                         \n"
"    \"license\",     \"GPL3+\",                                               \n"
"    \"description\", _(\"This plug-in uses the algorithm described by John \" \n"
"                     \"Schlag, \\\"Fast Embossing Effects on Raster Image \"  \n"
"                     \"Data\\\" in Graphics GEMS IV (ISBN 0-12-336155-9). \"  \n"
"                     \"It takes a drawable to be applied as a bump \"         \n"
"                     \"map to another image and produces a nice embossing \"  \n"
"                     \"effect.\"),                                            \n"
"    NULL);                                                                    \n"
"}                                                                             \n"
"                                                                              \n"
"#endif                                                                        \n"
;
