static const char* op_c_source =
"/* This file is an image processing operation for GEGL                        \n"
" *                                                                            \n"
" * GEGL is free software; you can redistribute it and/or                      \n"
" * modify it under the terms of the GNU Lesser General Public                 \n"
" * License as published by the Free Software Foundation; either               \n"
" * version 3 of the License, or (at your option) any later version.           \n"
" *                                                                            \n"
" * GEGL is distributed in the hope that it will be useful,                    \n"
" * but WITHOUT ANY WARRANTY; without even the implied warranty of             \n"
" * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU          \n"
" * Lesser General Public License for more details.                            \n"
" *                                                                            \n"
" * You should have received a copy of the GNU Lesser General Public           \n"
" * License along with GEGL; if not, see <http://www.gnu.org/licenses/>.       \n"
" *                                                                            \n"
" * Copyright 2008 Hans Petter Jansson <hpj@copyleft.no>                       \n"
" *           2012 Øyvind Kolås <pippin@gimp.org>                            \n"
" */                                                                           \n"
"                                                                              \n"
"#include \"config.h\"                                                         \n"
"#include <glib/gi18n-lib.h>                                                   \n"
"                                                                              \n"
"                                                                              \n"
"#ifdef GEGL_PROPERTIES                                                        \n"
"                                                                              \n"
"enum_start (gegl_dither_strategy)                                             \n"
"  enum_value (GEGL_DITHER_NONE,             \"none\",             N_(\"None\"))\n"
"  enum_value (GEGL_DITHER_RANDOM,           \"random\",           N_(\"Random\"))\n"
"  enum_value (GEGL_DITHER_RESILIENT,        \"resilient\",        N_(\"Resilient\"))\n"
"  enum_value (GEGL_DITHER_RANDOM_COVARIANT, \"random-covariant\", N_(\"Random Covariant\"))\n"
"  enum_value (GEGL_DITHER_BAYER,            \"bayer\",            N_(\"Bayer\"))\n"
"  enum_value (GEGL_DITHER_FLOYD_STEINBERG,  \"floyd-steinberg\",  N_(\"Floyd-Steinberg\"))\n"
"enum_end (GeglDitherStrategy)                                                 \n"
"                                                                              \n"
"property_int  (red_bits, _(\"Red bits\"), 8)                                  \n"
"    description(_(\"Number of bits for red channel\"))                        \n"
"    value_range (1, 16)                                                       \n"
"                                                                              \n"
"property_int  (green_bits, _(\"Green bits\"), 8)                              \n"
"    description(_(\"Number of bits for green channel\"))                      \n"
"    value_range (1, 16)                                                       \n"
"                                                                              \n"
"property_int  (blue_bits, _(\"Blue bits\"), 8)                                \n"
"    description(_(\"Number of bits for blue channel\"))                       \n"
"    value_range (1, 16)                                                       \n"
"                                                                              \n"
"property_int  (alpha_bits, _(\"Alpha bits\"), 8)                              \n"
"    description(_(\"Number of bits for alpha channel\"))                      \n"
"    value_range (1, 16)                                                       \n"
"                                                                              \n"
"property_enum (dither_strategy, _(\"Dithering strategy\"),                    \n"
"               GeglDitherStrategy, gegl_dither_strategy, GEGL_DITHER_RESILIENT)\n"
"    description (_(\"The dithering strategy to use\"))                        \n"
"                                                                              \n"
"property_seed (seed, _(\"Random seed\"), rand)                                \n"
"                                                                              \n"
"#else                                                                         \n"
"                                                                              \n"
"#define GEGL_OP_FILTER                                                        \n"
"#define GEGL_OP_C_SOURCE color-reduction.c                                    \n"
"                                                                              \n"
"#include \"gegl-op.h\"                                                        \n"
"                                                                              \n"
"#define REDUCE_16B(value) (((value) & ((1 << 17) - 1)) - 65536)               \n"
"                                                                              \n"
"static void                                                                   \n"
"prepare (GeglOperation *operation)                                            \n"
"{                                                                             \n"
"  gegl_operation_set_format (operation, \"input\", babl_format (\"R'G'B'A u16\"));\n"
"  gegl_operation_set_format (operation, \"output\", babl_format (\"R'G'B'A u16\"));\n"
"}                                                                             \n"
"                                                                              \n"
"static void                                                                   \n"
"generate_channel_masks (guint *channel_bits,                                  \n"
"                        guint *channel_mask)                                  \n"
"{                                                                             \n"
"  gint i;                                                                     \n"
"                                                                              \n"
"  for (i = 0; i < 4; i++)                                                     \n"
"    channel_mask [i] = ~((1 << (16 - channel_bits [i])) - 1);                 \n"
"}                                                                             \n"
"                                                                              \n"
"static guint                                                                  \n"
"quantize_value (guint value,                                                  \n"
"                guint n_bits,                                                 \n"
"                guint mask)                                                   \n"
"{                                                                             \n"
"  gint i;                                                                     \n"
"                                                                              \n"
"  value &= mask;                                                              \n"
"                                                                              \n"
"  for (i = n_bits; i < 16; i += n_bits)                                       \n"
"    value |= value >> i;                                                      \n"
"                                                                              \n"
"  return value;                                                               \n"
"}                                                                             \n"
"                                                                              \n"
"static void                                                                   \n"
"process_floyd_steinberg (GeglBuffer          *input,                          \n"
"                         GeglBuffer          *output,                         \n"
"                         const GeglRectangle *result,                         \n"
"                         guint               *channel_bits)                   \n"
"{                                                                             \n"
"  GeglRectangle  line_rect;                                                   \n"
"  guint16       *line_buf;                                                    \n"
"  gdouble       *error_buf [2];                                               \n"
"  guint          channel_mask [4];                                            \n"
"  gint           y;                                                           \n"
"                                                                              \n"
"  line_rect.x      = result->x;                                               \n"
"  line_rect.y      = result->y;                                               \n"
"  line_rect.width  = result->width;                                           \n"
"  line_rect.height = 1;                                                       \n"
"                                                                              \n"
"  line_buf      = g_new  (guint16, line_rect.width * 4);                      \n"
"  error_buf [0] = g_new0 (gdouble, line_rect.width * 4);                      \n"
"  error_buf [1] = g_new0 (gdouble, line_rect.width * 4);                      \n"
"                                                                              \n"
"  generate_channel_masks (channel_bits, channel_mask);                        \n"
"                                                                              \n"
"  for (y = 0; y < result->height; y++)                                        \n"
"    {                                                                         \n"
"      gdouble  *error_buf_swap;                                               \n"
"      gint      step;                                                         \n"
"      gint      start_x;                                                      \n"
"      gint      end_x;                                                        \n"
"      gint      x;                                                            \n"
"                                                                              \n"
"      /* Serpentine scanning; reverse direction every row */                  \n"
"                                                                              \n"
"      if (y & 1)                                                              \n"
"        {                                                                     \n"
"          start_x = result->width - 1;                                        \n"
"          end_x   = -1;                                                       \n"
"          step    = -1;                                                       \n"
"        }                                                                     \n"
"      else                                                                    \n"
"        {                                                                     \n"
"          start_x = 0;                                                        \n"
"          end_x   = result->width;                                            \n"
"          step    = 1;                                                        \n"
"        }                                                                     \n"
"                                                                              \n"
"      /* Pull input row */                                                    \n"
"                                                                              \n"
"      gegl_buffer_get (input, &line_rect, 1.0, babl_format (\"R'G'B'A u16\"), line_buf,\n"
"                       GEGL_AUTO_ROWSTRIDE, GEGL_ABYSS_NONE);                 \n"
"                                                                              \n"
"      /* Process the row */                                                   \n"
"                                                                              \n"
"      for (x = start_x; x != end_x; x += step)                                \n"
"        {                                                                     \n"
"          guint16  *pixel = &line_buf [x * 4];                                \n"
"          guint     ch;                                                       \n"
"                                                                              \n"
"          for (ch = 0; ch < 4; ch++)                                          \n"
"            {                                                                 \n"
"              gdouble value;                                                  \n"
"              gdouble value_clamped;                                          \n"
"              gdouble quantized;                                              \n"
"              gdouble qerror;                                                 \n"
"                                                                              \n"
"              value         = pixel [ch] + error_buf [0] [x * 4 + ch];        \n"
"              value_clamped = CLAMP (value, 0.0, 65535.0);                    \n"
"              quantized     = quantize_value ((guint) (value_clamped + 0.5), channel_bits [ch], channel_mask [ch]);\n"
"              qerror        = value - quantized;                              \n"
"                                                                              \n"
"              pixel [ch] = (guint16) quantized;                               \n"
"                                                                              \n"
"              /* Distribute the error */                                      \n"
"                                                                              \n"
"              error_buf [1] [x * 4 + ch] += qerror * 5.0 / 16.0;  /* Down */  \n"
"                                                                              \n"
"              if (x + step >= 0 && x + step < result->width)                  \n"
"                {                                                             \n"
"                  error_buf [0] [(x + step) * 4 + ch] += qerror * 6.0 / 16.0;  /* Ahead */\n"
"                  error_buf [1] [(x + step) * 4 + ch] += qerror * 1.0 / 16.0;  /* Down, ahead */\n"
"                }                                                             \n"
"                                                                              \n"
"              if (x - step >= 0 && x - step < result->width)                  \n"
"                {                                                             \n"
"                  error_buf [1] [(x - step) * 4 + ch] += qerror * 3.0 / 16.0;  /* Down, behind */\n"
"                }                                                             \n"
"            }                                                                 \n"
"        }                                                                     \n"
"                                                                              \n"
"      /* Swap error accumulation rows */                                      \n"
"                                                                              \n"
"      error_buf_swap = error_buf [0];                                         \n"
"      error_buf [0]  = error_buf [1];                                         \n"
"      error_buf [1]  = error_buf_swap;                                        \n"
"                                                                              \n"
"      /* Clear error buffer for next-plus-one line */                         \n"
"                                                                              \n"
"      memset (error_buf [1], 0, line_rect.width * 4 * sizeof (gdouble));      \n"
"                                                                              \n"
"      /* Push output row */                                                   \n"
"                                                                              \n"
"      gegl_buffer_set (output, &line_rect, 0, babl_format (\"R'G'B'A u16\"), line_buf, GEGL_AUTO_ROWSTRIDE);\n"
"      line_rect.y++;                                                          \n"
"    }                                                                         \n"
"                                                                              \n"
"  g_free (line_buf);                                                          \n"
"  g_free (error_buf [0]);                                                     \n"
"  g_free (error_buf [1]);                                                     \n"
"}                                                                             \n"
"                                                                              \n"
"static const gdouble bayer_matrix_8x8 [] =                                    \n"
"{                                                                             \n"
"   1, 49, 13, 61,  4, 52, 16, 64,                                             \n"
"  33, 17, 45, 29, 36, 20, 48, 32,                                             \n"
"   9, 57,  5, 53, 12, 60,  8, 56,                                             \n"
"  41, 25, 37, 21, 44, 28, 40, 24,                                             \n"
"   3, 51, 15, 63,  2, 50, 14, 62,                                             \n"
"  35, 19, 47, 31, 34, 18, 46, 30,                                             \n"
"  11, 59,  7, 55, 10, 58,  6, 54,                                             \n"
"  43, 27, 39, 23, 42, 26, 38, 22                                              \n"
"};                                                                            \n"
"                                                                              \n"
"static void                                                                   \n"
"process_row_bayer (GeglBufferIterator *gi,                                    \n"
"                   guint               channel_mask [4],                      \n"
"                   guint               channel_bits [4],                      \n"
"                   gint                y)                                     \n"
"{                                                                             \n"
"  guint16 *data_in  = (guint16*) gi->data [0];                                \n"
"  guint16 *data_out = (guint16*) gi->data [1];                                \n"
"  guint x;                                                                    \n"
"  for (x = 0; x < gi->roi->width; x++)                                        \n"
"    {                                                                         \n"
"      guint pixel = 4 * (gi->roi->width * y + x);                             \n"
"      guint ch;                                                               \n"
"                                                                              \n"
"      for (ch = 0; ch < 4; ch++)                                              \n"
"        {                                                                     \n"
"          gdouble bayer;                                                      \n"
"          gdouble value;                                                      \n"
"          gdouble value_clamped;                                              \n"
"          gdouble quantized;                                                  \n"
"                                                                              \n"
"          bayer         = bayer_matrix_8x8 [((gi->roi->y + y) % 8) * 8 + ((gi->roi->x + x) % 8)];\n"
"          bayer         = ((bayer - 32) * 65536.0 / 65.0) / (1 << (channel_bits [ch] - 1));\n"
"          value         = data_in [pixel + ch] + bayer;                       \n"
"          value_clamped = CLAMP (value, 0.0, 65535.0);                        \n"
"          quantized     = quantize_value ((guint) (value_clamped + 0.5),      \n"
"                                          channel_bits [ch],                  \n"
"                                          channel_mask [ch]);                 \n"
"                                                                              \n"
"          data_out [pixel + ch] = (guint16) quantized;                        \n"
"        }                                                                     \n"
"    }                                                                         \n"
"}                                                                             \n"
"                                                                              \n"
"static void                                                                   \n"
"process_row_random_covariant (GeglBufferIterator *gi,                         \n"
"                              guint               channel_mask [4],           \n"
"                              guint               channel_bits [4],           \n"
"                              gint                y,                          \n"
"                              GeglRandom         *rand)                       \n"
"{                                                                             \n"
"  guint16 *data_in  = (guint16*) gi->data [0];                                \n"
"  guint16 *data_out = (guint16*) gi->data [1];                                \n"
"  guint x;                                                                    \n"
"  for (x = 0; x < gi->roi->width; x++)                                        \n"
"    {                                                                         \n"
"      guint pixel = 4 * (gi->roi->width * y + x);                             \n"
"      guint ch;                                                               \n"
"      gint  r = REDUCE_16B (gegl_random_int (rand, gi->roi->x + x,            \n"
"                                             gi->roi->y + y, 0, 0));          \n"
"      for (ch = 0; ch < 4; ch++)                                              \n"
"        {                                                                     \n"
"          gdouble value;                                                      \n"
"          gdouble value_clamped;                                              \n"
"          gdouble quantized;                                                  \n"
"                                                                              \n"
"          value         = data_in [pixel + ch] + (r / (1 << channel_bits [ch]));\n"
"          value_clamped = CLAMP (value, 0.0, 65535.0);                        \n"
"          quantized     = quantize_value ((guint) (value_clamped + 0.5),      \n"
"                                          channel_bits [ch],                  \n"
"                                          channel_mask [ch]);                 \n"
"                                                                              \n"
"          data_out [pixel + ch] = (guint16) quantized;                        \n"
"        }                                                                     \n"
"    }                                                                         \n"
"}                                                                             \n"
"                                                                              \n"
"static void                                                                   \n"
"process_row_random (GeglBufferIterator *gi,                                   \n"
"                    guint               channel_mask [4],                     \n"
"                    guint               channel_bits [4],                     \n"
"                    gint                y,                                    \n"
"                    GeglRandom         *rand)                                 \n"
"{                                                                             \n"
"  guint16 *data_in  = (guint16*) gi->data [0];                                \n"
"  guint16 *data_out = (guint16*) gi->data [1];                                \n"
"  guint x;                                                                    \n"
"  for (x = 0; x < gi->roi->width; x++)                                        \n"
"    {                                                                         \n"
"      guint pixel = 4 * (gi->roi->width * y + x);                             \n"
"      guint ch;                                                               \n"
"      for (ch = 0; ch < 4; ch++)                                              \n"
"        {                                                                     \n"
"          gdouble value;                                                      \n"
"          gdouble value_clamped;                                              \n"
"          gdouble quantized;                                                  \n"
"          gint    r = REDUCE_16B (gegl_random_int (rand, gi->roi->x + x,      \n"
"                                                   gi->roi->y + y, 0, ch));   \n"
"                                                                              \n"
"          value         = data_in [pixel + ch] + (r / (1 << channel_bits [ch]));\n"
"          value_clamped = CLAMP (value, 0.0, 65535.0);                        \n"
"          quantized     = quantize_value ((guint) (value_clamped + 0.5),      \n"
"                                          channel_bits [ch],                  \n"
"                                          channel_mask [ch]);                 \n"
"                                                                              \n"
"          data_out [pixel + ch] = (guint16) quantized;                        \n"
"        }                                                                     \n"
"    }                                                                         \n"
"}                                                                             \n"
"                                                                              \n"
"static void                                                                   \n"
"process_row_resilient (GeglBufferIterator *gi,                                \n"
"                       guint               channel_mask [4],                  \n"
"                       guint               channel_bits [4],                  \n"
"                       gint                y,                                 \n"
"                       GeglRandom         *rand)                              \n"
"{                                                                             \n"
"  guint16 *data_in  = (guint16*) gi->data [0];                                \n"
"  guint16 *data_out = (guint16*) gi->data [1];                                \n"
"  guint    x;                                                                 \n"
"  for (x = 0; x < gi->roi->width; x++)                                        \n"
"    {                                                                         \n"
"      guint pixel = 4 * (gi->roi->width * y + x);                             \n"
"      guint ch;                                                               \n"
"      for (ch = 0; ch < 4; ch++)                                              \n"
"        {                                                                     \n"
"          gdouble value;                                                      \n"
"          gdouble value_clamped;                                              \n"
"          gdouble quantized;                                                  \n"
"          gint    r = REDUCE_16B (gegl_random_int (rand, gi->roi->x + x,      \n"
"                                                   gi->roi->y + y, 0, ch));   \n"
"          value         = data_in [pixel + ch];                               \n"
"          value         = value + ((65535.0 / (8 * value + 48 * 65535)) + 1.2) *\n"
"                                  (r / (1 << channel_bits [ch]));             \n"
"          value_clamped = CLAMP (value, 0.0, 65535.0);                        \n"
"          quantized     = quantize_value ((guint) (value_clamped + 0.5),      \n"
"                                          channel_bits [ch],                  \n"
"                                          channel_mask [ch]);                 \n"
"                                                                              \n"
"          data_out [pixel + ch] = (guint16) quantized;                        \n"
"        }                                                                     \n"
"    }                                                                         \n"
"}                                                                             \n"
"                                                                              \n"
"static void                                                                   \n"
"process_row_no_dither (GeglBufferIterator *gi,                                \n"
"                       guint               channel_mask [4],                  \n"
"                       guint               channel_bits [4],                  \n"
"                       guint               y)                                 \n"
"{                                                                             \n"
"  guint16 *data_in  = (guint16*) gi->data [0];                                \n"
"  guint16 *data_out = (guint16*) gi->data [1];                                \n"
"  guint x;                                                                    \n"
"  for (x = 0; x < gi->roi->width; x++)                                        \n"
"    {                                                                         \n"
"      guint pixel = 4 * (gi->roi->width * y + x);                             \n"
"      guint ch;                                                               \n"
"      for (ch = 0; ch < 4; ch++)                                              \n"
"        {                                                                     \n"
"          data_out [pixel + ch] = (guint16) quantize_value (data_in [pixel + ch],\n"
"                                                            channel_bits [ch],\n"
"                                                            channel_mask [ch]);\n"
"        }                                                                     \n"
"    }                                                                         \n"
"}                                                                             \n"
"                                                                              \n"
"static void                                                                   \n"
"process_standard (GeglBuffer          *input,                                 \n"
"                  GeglBuffer          *output,                                \n"
"                  const GeglRectangle *result,                                \n"
"                  guint               *channel_bits,                          \n"
"                  GeglRandom          *rand,                                  \n"
"                  GeglDitherStrategy   dither_strategy)                       \n"
"{                                                                             \n"
"  GeglBufferIterator *gi;                                                     \n"
"  guint               channel_mask [4];                                       \n"
"                                                                              \n"
"  generate_channel_masks (channel_bits, channel_mask);                        \n"
"                                                                              \n"
"  gi = gegl_buffer_iterator_new (input, result, 0, babl_format (\"R'G'B'A u16\"),\n"
"                                 GEGL_ACCESS_READ, GEGL_ABYSS_NONE);          \n"
"                                                                              \n"
"  gegl_buffer_iterator_add (gi, output, result, 0, babl_format (\"R'G'B'A u16\"),\n"
"                            GEGL_ACCESS_WRITE, GEGL_ABYSS_NONE);              \n"
"                                                                              \n"
"  while (gegl_buffer_iterator_next (gi))                                      \n"
"    {                                                                         \n"
"      guint    y;                                                             \n"
"      for (y = 0; y < gi->roi->height; y++)                                   \n"
"        {                                                                     \n"
"          switch (dither_strategy)                                            \n"
"            {                                                                 \n"
"            case GEGL_DITHER_NONE:                                            \n"
"              process_row_no_dither (gi, channel_mask, channel_bits, y);      \n"
"              break;                                                          \n"
"            case GEGL_DITHER_RANDOM:                                          \n"
"              process_row_random (gi, channel_mask, channel_bits, y, rand);   \n"
"              break;                                                          \n"
"            case GEGL_DITHER_RESILIENT:                                       \n"
"              process_row_resilient (gi, channel_mask, channel_bits, y, rand);\n"
"              break;                                                          \n"
"            case GEGL_DITHER_RANDOM_COVARIANT:                                \n"
"              process_row_random_covariant (gi, channel_mask, channel_bits, y, rand);\n"
"              break;                                                          \n"
"            case GEGL_DITHER_BAYER:                                           \n"
"              process_row_bayer (gi, channel_mask, channel_bits, y);          \n"
"              break;                                                          \n"
"            case GEGL_DITHER_FLOYD_STEINBERG:                                 \n"
"              /* Done separately */                                           \n"
"              break;                                                          \n"
"            default:                                                          \n"
"              process_row_no_dither (gi, channel_mask, channel_bits, y);      \n"
"            }                                                                 \n"
"        }                                                                     \n"
"    }                                                                         \n"
"}                                                                             \n"
"                                                                              \n"
"static GeglRectangle                                                          \n"
"get_required_for_output (GeglOperation       *self,                           \n"
"                         const gchar         *input_pad,                      \n"
"                         const GeglRectangle *roi)                            \n"
"{                                                                             \n"
"  GeglProperties *o = GEGL_PROPERTIES (self);                                 \n"
"                                                                              \n"
"  if (o->dither_strategy == GEGL_DITHER_FLOYD_STEINBERG)                      \n"
"    return *gegl_operation_source_get_bounding_box (self, \"input\");         \n"
"  else                                                                        \n"
"    return *roi;                                                              \n"
"}                                                                             \n"
"                                                                              \n"
"static GeglRectangle                                                          \n"
"get_cached_region (GeglOperation       *self,                                 \n"
"                   const GeglRectangle *roi)                                  \n"
"{                                                                             \n"
"  GeglProperties *o = GEGL_PROPERTIES (self);                                 \n"
"                                                                              \n"
"  if (o->dither_strategy == GEGL_DITHER_FLOYD_STEINBERG)                      \n"
"    return *gegl_operation_source_get_bounding_box (self, \"input\");         \n"
"  else                                                                        \n"
"    return *roi;                                                              \n"
"}                                                                             \n"
"                                                                              \n"
"static gboolean                                                               \n"
"process (GeglOperation       *operation,                                      \n"
"         GeglBuffer          *input,                                          \n"
"         GeglBuffer          *output,                                         \n"
"         const GeglRectangle *result,                                         \n"
"         gint                 level)                                          \n"
"{                                                                             \n"
"  GeglProperties *o = GEGL_PROPERTIES (operation);                            \n"
"  guint       channel_bits [4];                                               \n"
"                                                                              \n"
"  channel_bits [0] = o->red_bits;                                             \n"
"  channel_bits [1] = o->green_bits;                                           \n"
"  channel_bits [2] = o->blue_bits;                                            \n"
"  channel_bits [3] = o->alpha_bits;                                           \n"
"                                                                              \n"
"  if (o->dither_strategy != GEGL_DITHER_FLOYD_STEINBERG)                      \n"
"    process_standard (input, output, result, channel_bits,                    \n"
"                      o->rand, o->dither_strategy);                           \n"
"  else                                                                        \n"
"    process_floyd_steinberg (input, output, result, channel_bits);            \n"
"                                                                              \n"
"  return TRUE;                                                                \n"
"}                                                                             \n"
"                                                                              \n"
"static void                                                                   \n"
"gegl_op_class_init (GeglOpClass *klass)                                       \n"
"{                                                                             \n"
"  GeglOperationClass       *operation_class;                                  \n"
"  GeglOperationFilterClass *filter_class;                                     \n"
"  gchar                    *composition = \"<?xml version='1.0' encoding='UTF-8'?>\"\n"
"    \"<gegl>\"                                                                \n"
"    \"<node operation='gegl:color-reduction'>\"                               \n"
"    \"  <params>\"                                                            \n"
"    \"    <param name='red-bits'>2</param>\"                                  \n"
"    \"    <param name='green-bits'>2</param>\"                                \n"
"    \"    <param name='blue-bits'>2</param>\"                                 \n"
"    \"    <param name='alpha-bits'>2</param>\"                                \n"
"    \"    <param name='dither-strategy'>floyd-steinberg</param>\"             \n"
"    \"  </params>\"                                                           \n"
"    \"</node>\"                                                               \n"
"    \"<node operation='gegl:load'>\"                                          \n"
"    \"  <params>\"                                                            \n"
"    \"    <param name='path'>standard-input.png</param>\"                     \n"
"    \"  </params>\"                                                           \n"
"    \"</node>\"                                                               \n"
"    \"</gegl>\";                                                              \n"
"                                                                              \n"
"  operation_class = GEGL_OPERATION_CLASS (klass);                             \n"
"  filter_class    = GEGL_OPERATION_FILTER_CLASS (klass);                      \n"
"                                                                              \n"
"  operation_class->prepare = prepare;                                         \n"
"  operation_class->get_required_for_output = get_required_for_output;         \n"
"  operation_class->get_cached_region = get_cached_region;                     \n"
"  filter_class->process = process;                                            \n"
"                                                                              \n"
"  gegl_operation_class_set_keys (operation_class,                             \n"
"    \"name\",        \"gegl:color-reduction\",                                \n"
"    \"title\",       _(\"Color Reduction\"),                                  \n"
"    \"categories\",  \"dither\",                                              \n"
"    \"description\", _(\"Reduce the number of colors in the image, by reducing \"\n"
"                     \"the bits per channel (colors and alpha), different dithering methods\"\n"
"                     \"can be specified to counteract quantization induced banding.\"),\n"
"    \"reference-composition\", composition,                                   \n"
"    NULL);                                                                    \n"
"}                                                                             \n"
"                                                                              \n"
"#endif                                                                        \n"
;
